"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const index_1 = require("../index");
describe('serialize', () => {
    describe('encodeDate', () => {
        it('produces the correct value', () => {
            const date = new Date(2016, 2, 1);
            const result = index_1.encodeDate(date);
            expect(result).toBe('2016-03-01');
        });
        it('handles null and undefined', () => {
            const result = index_1.encodeDate(null);
            expect(result).toBeUndefined();
            expect(index_1.encodeDate(undefined)).not.toBeDefined();
        });
    });
    describe('decodeDate', () => {
        it('produces the correct value', () => {
            let result = index_1.decodeDate('2016-03-01');
            // result is a Date object
            expect(result.getFullYear()).toBe(2016);
            expect(result.getMonth()).toBe(2);
            expect(result.getDate()).toBe(1);
            // javascript likes to give us 2015-12-31 19:00, so test this doesn't.
            result = index_1.decodeDate('2016');
            expect(result.getFullYear()).toBe(2016);
            expect(result.getMonth()).toBe(0);
            expect(result.getDate()).toBe(1);
        });
        it('handles null and undefined', () => {
            const result = index_1.decodeDate(null);
            expect(result).not.toBeDefined();
            expect(index_1.decodeDate(undefined)).not.toBeDefined();
            expect(index_1.decodeDate('')).not.toBeDefined();
        });
        it('handles malformed input', () => {
            const result = index_1.decodeDate('foo-one-two');
            expect(result).not.toBeDefined();
        });
    });
    describe('encodeBoolean', () => {
        it('produces the correct value', () => {
            expect(index_1.encodeBoolean(true)).toBe('1');
            expect(index_1.encodeBoolean(false)).toBe('0');
            expect(index_1.encodeBoolean(undefined)).not.toBeDefined();
        });
    });
    describe('decodeBoolean', () => {
        it('produces the correct value', () => {
            expect(index_1.decodeBoolean('1')).toBe(true);
            expect(index_1.decodeBoolean('0')).toBe(false);
            expect(index_1.decodeBoolean(undefined)).not.toBeDefined();
            expect(index_1.decodeBoolean('')).not.toBeDefined();
        });
        it('handles malformed input', () => {
            expect(index_1.decodeBoolean('foo')).not.toBeDefined();
        });
    });
    describe('encodeNumber', () => {
        it('produces the correct value', () => {
            expect(index_1.encodeNumber(123)).toBe('123');
            expect(index_1.encodeNumber(-32.12)).toBe('-32.12');
            expect(index_1.encodeNumber(undefined)).not.toBeDefined();
        });
    });
    describe('decodeNumber', () => {
        it('produces the correct value', () => {
            expect(index_1.decodeNumber('99')).toBe(99);
            expect(index_1.decodeNumber('-58.21')).toBe(-58.21);
            expect(index_1.decodeNumber(undefined)).not.toBeDefined();
            expect(index_1.decodeNumber('')).not.toBeDefined();
        });
        it('handles malformed input', () => {
            expect(index_1.decodeNumber('foo')).not.toBeDefined();
        });
    });
    describe('encodeString', () => {
        it('produces the correct value', () => {
            expect(index_1.encodeString('foo')).toBe('foo');
            expect(index_1.encodeString(undefined)).not.toBeDefined();
        });
    });
    describe('decodeString', () => {
        it('produces the correct value', () => {
            expect(index_1.decodeString('bar')).toBe('bar');
            expect(index_1.decodeString('')).toBe('');
            expect(index_1.decodeString(undefined)).not.toBeDefined();
            expect(index_1.decodeString(null)).not.toBeDefined();
        });
    });
    describe('encodeJson', () => {
        it('produces the correct value', () => {
            const input = { test: '123', foo: [1, 2, 3] };
            expect(index_1.encodeJson(input)).toBe(JSON.stringify(input));
            expect(index_1.encodeJson(undefined)).not.toBeDefined();
            expect(index_1.encodeJson(null)).not.toBeDefined();
            expect(index_1.encodeJson(0)).toBe('0');
            expect(index_1.encodeJson(false)).toBe('false');
        });
    });
    describe('decodeJson', () => {
        it('produces the correct value', () => {
            const output = index_1.decodeJson('{"foo": "bar", "jim": ["grill"]}');
            const expectedOutput = {
                foo: 'bar',
                jim: ['grill'],
            };
            expect(output).toEqual(expectedOutput);
            expect(index_1.decodeJson(undefined)).not.toBeDefined();
            expect(index_1.decodeJson('')).not.toBeDefined();
        });
        it('handles malformed input', () => {
            expect(index_1.decodeJson('foo')).not.toBeDefined();
        });
    });
    describe('encodeArray', () => {
        it('produces the correct value', () => {
            const input = ['a', 'b', 'c'];
            expect(index_1.encodeArray(input)).toBe('a_b_c');
            expect(index_1.encodeArray(undefined)).not.toBeDefined();
        });
    });
    describe('decodeArray', () => {
        it('produces the correct value', () => {
            const output = index_1.decodeArray('a_b_c');
            const expectedOutput = ['a', 'b', 'c'];
            expect(output).toEqual(expectedOutput);
            expect(index_1.decodeArray(undefined)).not.toBeDefined();
            expect(index_1.decodeArray('')).not.toBeDefined();
        });
        it('filters empty values', () => {
            expect(index_1.decodeArray('__')).toEqual([]);
        });
    });
    describe('encodeObject', () => {
        it('produces the correct value', () => {
            const input = { test: 'bar', foo: '94' };
            const expectedOutput = 'test-bar_foo-94';
            expect(index_1.encodeObject(input, '-', '_')).toBe(expectedOutput);
            expect(index_1.encodeObject(undefined)).not.toBeDefined();
            expect(index_1.encodeObject({})).not.toBeDefined();
        });
    });
    describe('decodeObject', () => {
        it('produces the correct value', () => {
            const output = index_1.decodeObject('foo-bar_jim-grill_iros-91');
            const expectedOutput = {
                foo: 'bar',
                jim: 'grill',
                iros: '91',
            };
            expect(output).toEqual(expectedOutput);
            expect(index_1.decodeObject(undefined)).not.toBeDefined();
            expect(index_1.decodeObject('')).not.toBeDefined();
        });
        it('handles malformed input', () => {
            expect(index_1.decodeObject('foo-bar-jim-grill')).toEqual({ foo: 'bar' });
            expect(index_1.decodeObject('foo_bar_jim_grill')).toEqual({
                foo: undefined,
                bar: undefined,
                jim: undefined,
                grill: undefined,
            });
        });
    });
    describe('encodeNumericArray', () => {
        it('produces the correct value', () => {
            const input = [9, 4, 0];
            expect(index_1.encodeNumericArray(input)).toBe('9_4_0');
            expect(index_1.encodeNumericArray(undefined)).not.toBeDefined();
        });
    });
    describe('decodeNumericArray', () => {
        it('produces the correct value', () => {
            const output = index_1.decodeNumericArray('9_4_0');
            const expectedOutput = [9, 4, 0];
            expect(output).toEqual(expectedOutput);
            expect(index_1.decodeNumericArray(undefined)).not.toBeDefined();
            expect(index_1.decodeNumericArray('')).not.toBeDefined();
        });
        it('filters empty values', () => {
            expect(index_1.decodeNumericArray('__')).toEqual([]);
        });
    });
    describe('encodeNumericObject', () => {
        it('produces the correct value', () => {
            const input = { test: 55, foo: 94 };
            const expectedOutput = 'test-55_foo-94';
            expect(index_1.encodeNumericObject(input, '-', '_')).toBe(expectedOutput);
            expect(index_1.encodeNumericObject(undefined)).not.toBeDefined();
            expect(index_1.encodeNumericObject({})).not.toBeDefined();
        });
    });
    describe('decodeNumericObject', () => {
        it('produces the correct value', () => {
            const output = index_1.decodeNumericObject('foo-55_jim-100_iros-94');
            const expectedOutput = {
                foo: 55,
                jim: 100,
                iros: 94,
            };
            expect(output).toEqual(expectedOutput);
            expect(index_1.decodeNumericObject(undefined)).not.toBeDefined();
            expect(index_1.decodeNumericObject('')).not.toBeDefined();
        });
        it('handles malformed input', () => {
            expect(index_1.decodeNumericObject('foo-bar-jim-grill')).toEqual({
                foo: undefined,
            });
            expect(index_1.decodeNumericObject('foo_bar_jim_grill')).toEqual({
                foo: undefined,
                bar: undefined,
                jim: undefined,
                grill: undefined,
            });
        });
    });
});
