/**
 * Encodes a date as a string in YYYY-MM-DD format.
 *
 * @param {Date} date
 * @return {String} the encoded date
 */
export declare function encodeDate(date: Date | null | undefined): string | undefined;
/**
 * Converts a date in the format 'YYYY-mm-dd...' into a proper date, because
 * new Date() does not do that correctly. The date can be as complete or incomplete
 * as necessary (aka, '2015', '2015-10', '2015-10-01').
 * It will not work for dates that have times included in them.
 *
 * @param  {String} dateString String date form like '2015-10-01'
 * @return {Date} parsed date
 */
export declare function decodeDate(dateString: string | null | undefined): Date | undefined;
/**
 * Encodes a boolean as a string. true -> "1", false -> "0".
 *
 * @param {Boolean} bool
 * @return {String} the encoded boolean
 */
export declare function encodeBoolean(bool: boolean | null | undefined): string | undefined;
/**
 * Decodes a boolean from a string. "1" -> true, "0" -> false.
 * Everything else maps to undefined.
 *
 * @param {String} boolStr the encoded boolean string
 * @return {Boolean} the boolean value
 */
export declare function decodeBoolean(boolStr: string | null | undefined): boolean | undefined;
/**
 * Encodes a number as a string.
 *
 * @param {Number} num
 * @return {String} the encoded number
 */
export declare function encodeNumber(num: number | null | undefined): string | undefined;
/**
 * Decodes a number from a string via parseFloat. If the number is invalid,
 * it returns undefined.
 *
 * @param {String} numStr the encoded number string
 * @return {Number} the number value
 */
export declare function decodeNumber(numStr: string | null | undefined): number | undefined;
/**
 * Encodes a string while safely handling null and undefined values.
 *
 * @param {String} string
 * @return {String} the encoded string
 */
export declare function encodeString(str: string | null | undefined): string | undefined;
/**
 * Decodes a string while safely handling null and undefined values.
 *
 * @param {String} str the encoded string
 * @return {String} the string value
 */
export declare function decodeString(str: string | null | undefined): string | undefined;
/**
 * Encodes anything as a JSON string.
 *
 * @param {Any} any The thing to be encoded
 * @return {String} The JSON string representation of any
 */
export declare function encodeJson(any: any | null | undefined): string | undefined;
/**
 * Decodes a JSON string into javascript
 *
 * @param {String} jsonStr The JSON string representation
 * @return {Any} The javascript representation
 */
export declare function decodeJson(jsonStr: string | null | undefined): any | undefined;
/**
 * Encodes an array as a JSON string.
 *
 * @param {Array} array The array to be encoded
 * @return {String} The JSON string representation of array
 */
export declare function encodeArray(array: string[] | null | undefined, entrySeparator?: string): string | undefined;
/**
 * Decodes a JSON string into javascript array
 *
 * @param {String} jsonStr The JSON string representation
 * @return {Array} The javascript representation
 */
export declare function decodeArray(arrayStr: string | null | undefined, entrySeparator?: string): string[] | undefined;
/**
 * Encodes a numeric array as a JSON string. (alias of encodeArray)
 *
 * @param {Array} array The array to be encoded
 * @return {String} The JSON string representation of array
 */
export declare const encodeNumericArray: (array: number[] | null | undefined, entrySeparator?: string | undefined) => string | undefined;
/**
 * Decodes a JSON string into javascript array where all entries are numbers
 *
 * @param {String} jsonStr The JSON string representation
 * @return {Array} The javascript representation
 */
export declare function decodeNumericArray(arrayStr: string | null | undefined, entrySeparator?: string): number[] | undefined;
/**
 * Encode simple objects as readable strings. Currently works only for simple,
 * flat objects where values are numbers, booleans or strings.
 *
 * For example { foo: bar, boo: baz } -> "foo-bar_boo-baz"
 *
 * @param {Object} object The object to encode
 * @param {String} keyValSeparator="-" The separator between keys and values
 * @param {String} entrySeparator="_" The separator between entries
 * @return {String} The encoded object
 */
export declare function encodeObject(obj: {
    [key: string]: string | undefined;
} | null | undefined, keyValSeparator?: string, entrySeparator?: string): string | undefined;
/**
 * Decodes a simple object to javascript. Currently works only for simple,
 * flat objects where values are numbers, booleans or strings.
 *
 * For example "foo-bar_boo-baz" -> { foo: bar, boo: baz }
 *
 * @param {String} objStr The object string to decode
 * @param {String} keyValSeparator="-" The separator between keys and values
 * @param {String} entrySeparator="_" The separator between entries
 * @return {Object} The javascript object
 */
export declare function decodeObject(objStr: string | null | undefined, keyValSeparator?: string, entrySeparator?: string): {
    [key: string]: string | undefined;
} | undefined;
/**
 * Encode simple objects as readable strings. Alias of encodeObject.
 *
 * For example { foo: 123, boo: 521 } -> "foo-123_boo-521"
 *
 * @param {Object} object The object to encode
 * @param {String} keyValSeparator="-" The separator between keys and values
 * @param {String} entrySeparator="_" The separator between entries
 * @return {String} The encoded object
 */
export declare const encodeNumericObject: (obj: {
    [key: string]: number | undefined;
} | null | undefined, keyValSeparator?: string | undefined, entrySeparator?: string | undefined) => string | undefined;
/**
 * Decodes a simple object to javascript where all values are numbers.
 * Currently works only for simple, flat objects.
 *
 * For example "foo-123_boo-521" -> { foo: 123, boo: 521 }
 *
 * @param {String} objStr The object string to decode
 * @param {String} keyValSeparator="-" The separator between keys and values
 * @param {String} entrySeparator="_" The separator between entries
 * @return {Object} The javascript object
 */
export declare function decodeNumericObject(objStr: string | null | undefined, keyValSeparator?: string, entrySeparator?: string): {
    [key: string]: number | undefined;
} | undefined;
